#include "global.h"
#include "world.h"
#include "clipping.h"

/*
* Game class
* Controls the world, rendering, key presses etc
*/
class Game
{
      private: 
          // true while running
          bool running;
          // tour
          bool tourRunning;
          float tourPeriod;
          MotionPath tourPosPath;
          MotionPath tourObjPath;
          // the world simulation
          World* world;
          // camera
          CameraView camView; // position and view
          GLfloat camFwdV; // forward velocity along z axis in m per s
          GLfloat camFwdA; // acceleration of camera, while key depressed
          GLfloat camHorizAV; // angluar velocity (deg per s) in xz plane when turning
          GLfloat camVertAV; // anglular v (deg per s) in the zy plane when turning
          // update camera orientation 
          void camUpdate(GLfloat dT);
          // move along the tour path
          void tourStep(GLfloat dT);
          // move to predefined screenshot location
          void viewPicture();
          // shows help file
          void showHelp();
          // handles SDL events
          void handleEvent(SDL_Event* event);
          // get time
          GLfloat getTime() {
              return ((GLfloat)SDL_GetTicks()) / 1000.0;
          }
      public:
          // constructor
          Game();
          // destructor
          ~Game();
          // start game running (main loop)
          void run();
          // exit
          void exit();
          // start and stop automatic tour
          void startTour();
          void stopTour();
          // reset
          void reset();
};
