#include "regulargrid.h"

/*
* TerrainGrid class
* ----------------------
* Creates a random terrain using the 
* diamond-square fractal algorithm.
* -------------------------
* Tristan Aubrey-Jones 18/12/2007
*/

class TerrainGrid: public RegularGrid
{
      private:
              // height (as normals created in advance to create in
              // the unit square)
              float height;
              // roughness constant
              float roughness;
              // the dimension exponent e, where dimensions = (2^e,2^e)
              int e;
              // performs the square step of the diamond-square algorithm
              void performSquareStep(int sqrSize, RegularGridVertex ran);
              // peforms the diamond step of the diamond-square algorithm
              void performDiamondStep(int sqrSize, RegularGridVertex ran);
      public:
             // constructor
             // res - resolution exponent 
             // e.g. (1 gives 2x2, 8 gives 256x256)
             TerrainGrid(int res, float roughness, Vec3 v, Texture* tex): 
                 RegularGrid((2<<res)+1, (2<<res)+1, v, tex), e(res), 
                 height(v.y), roughness(roughness) {}
             // clears terrain to all -1, ready to be 
             // seeded, and then the terrain to be
             // generated
             void clear() { 
                 RegularGridVertex v;
                 v.y = -1.0;
                 setAll(v);
             };
             // sets height values in the grid, for use when generating
             // the random terrain.
             void setHeight(int x, int z, GLfloat h) {
                  RegularGridVertex v = get(x,z);
                  v.y = h;
                  set(x,z,v);
             }
             // seed height using float values in range 0..1
             void seedHeight(GLfloat x, GLfloat z, GLfloat h) {
                  GLfloat j = (float)(2 << e);
                  setHeight((int)(x*j), (int)(z*j), h);
             }
             // uses the seed values already in the grid 
             // to create a random terrain.
             void generate();
             // check for collision, and if
             // one occurs provides the point of
             // collision
             // position - vertex to check
             // direction - direction of vertex as it approached
             // result - when returns true contains the point of collision
             bool collisionDetect(Vec3* position, Vec3* direction, Vec3* result);
};

